@@ -1,6 +1,7 @@
 local util   = require 'utility'
 local define = require 'proto.define'
 local timer  = require 'timer'
+local scope  = require 'workspace.scope'
 
 ---@alias config.source '"client"'|'"path"'|'"local"'
 
@@ -46,7 +47,7 @@ end)
 
 register('Integer', 0, function (self, v)
     return type(v) == 'number'
-end,function (self, v)
+end, function (self, v)
     return math.floor(v)
 end)
 
@@ -117,7 +118,7 @@ end, function (self, value)
     end
     if type(value) == 'string' then
         local t = {}
-        for s in value:gmatch('[^'..self.sep..']+') do
+        for s in value:gmatch('[^' .. self.sep .. ']+') do
             t[s] = true
         end
         return t
@@ -128,7 +129,7 @@ end, function (self, subkey, subvalue, sep)
     self.sep      = sep
 end)
 
-register('Or', {}, function (self, value)
+register('Or', nil, function (self, value)
     for _, sub in ipairs(self.subs) do
         if sub:checker(value) then
             return true
@@ -142,7 +143,7 @@ end, function (self, value)
         end
     end
 end, function (self, ...)
-    self.subs = {...}
+    self.subs = { ... }
 end)
 
 local Template = {
@@ -151,6 +152,7 @@ local Template = {
                                                 "?.lua",
                                                 "?/init.lua",
                                             },
+    ['Lua.runtime.pathStrict']              = Type.Boolean >> false,
     ['Lua.runtime.special']                 = Type.Hash(Type.String, Type.String),
     ['Lua.runtime.meta']                    = Type.String >> '${version} ${language} ${encoding}',
     ['Lua.runtime.unicodeName']             = Type.Boolean,
@@ -165,15 +167,15 @@ local Template = {
                                             >> util.deepCopy(define.DiagnosticDefaultSeverity),
     ['Lua.diagnostics.neededFileStatus']    = Type.Hash(Type.String, Type.String)
                                             >> util.deepCopy(define.DiagnosticDefaultNeededFileStatus),
-    ['Lua.diagnostics.workspaceDelay']      = Type.Integer >> 0,
+    ['Lua.diagnostics.workspaceDelay']      = Type.Integer >> 5,
     ['Lua.diagnostics.workspaceRate']       = Type.Integer >> 100,
     ['Lua.diagnostics.libraryFiles']        = Type.String  >> 'Opened',
     ['Lua.diagnostics.ignoredFiles']        = Type.String  >> 'Opened',
-    ['Lua.workspace.ignoreDir']             = Type.Hash(Type.String, Type.Boolean, ';'),
+    ['Lua.workspace.ignoreDir']             = Type.Array(Type.String),
     ['Lua.workspace.ignoreSubmodules']      = Type.Boolean >> true,
     ['Lua.workspace.useGitIgnore']          = Type.Boolean >> true,
-    ['Lua.workspace.maxPreload']            = Type.Integer >> 1000,
-    ['Lua.workspace.preloadFileSize']       = Type.Integer >> 100,
+    ['Lua.workspace.maxPreload']            = Type.Integer >> 3000,
+    ['Lua.workspace.preloadFileSize']       = Type.Integer >> 500,
     ['Lua.workspace.library']               = Type.Hash(Type.String, Type.Boolean, ';'),
     ['Lua.workspace.checkThirdParty']       = Type.Boolean >> true,
     ['Lua.workspace.userThirdParty']        = Type.Array(Type.String),
@@ -186,6 +188,7 @@ local Template = {
     ['Lua.completion.autoRequire']          = Type.Boolean >> true,
     ['Lua.completion.showParams']           = Type.Boolean >> true,
     ['Lua.completion.requireSeparator']     = Type.String  >> '.',
+    ['Lua.completion.postfix']              = Type.String  >> '@',
     ['Lua.signatureHelp.enable']            = Type.Boolean >> true,
     ['Lua.hover.enable']                    = Type.Boolean >> true,
     ['Lua.hover.viewString']                = Type.Boolean >> true,
@@ -193,55 +196,118 @@ local Template = {
     ['Lua.hover.viewNumber']                = Type.Boolean >> true,
     ['Lua.hover.previewFields']             = Type.Integer >> 20,
     ['Lua.hover.enumsLimit']                = Type.Integer >> 5,
-    ['Lua.color.mode']                      = Type.String  >> 'Semantic',
+    ['Lua.semantic.enable']                 = Type.Boolean >> true,
+    ['Lua.semantic.variable']               = Type.Boolean >> true,
+    ['Lua.semantic.annotation']             = Type.Boolean >> true,
+    ['Lua.semantic.keyword']                = Type.Boolean >> false,
     ['Lua.hint.enable']                     = Type.Boolean >> false,
     ['Lua.hint.paramType']                  = Type.Boolean >> true,
     ['Lua.hint.setType']                    = Type.Boolean >> false,
     ['Lua.hint.paramName']                  = Type.String  >> 'All',
+    ['Lua.hint.await']                      = Type.Boolean >> true,
+    ['Lua.hint.arrayIndex']                 = Type.Boolean >> 'Auto',
     ['Lua.window.statusBar']                = Type.Boolean >> true,
     ['Lua.window.progressBar']              = Type.Boolean >> true,
-    ['Lua.telemetry.enable']                = Type.Or(Type.Boolean >> false, Type.Nil),
+    ['Lua.format.enable']                   = Type.Boolean >> true,
+    ['Lua.format.defaultConfig']            = Type.Hash(Type.String, Type.String)
+                                            >> {},
+    ['Lua.IntelliSense.traceLocalSet']      = Type.Boolean >> false,
+    ['Lua.IntelliSense.traceReturn']        = Type.Boolean >> false,
+    ['Lua.IntelliSense.traceBeSetted']      = Type.Boolean >> false,
+    ['Lua.IntelliSense.traceFieldInject']   = Type.Boolean >> false,
+    ['Lua.telemetry.enable']                = Type.Or(Type.Boolean >> false, Type.Nil) >> nil,
     ['files.associations']                  = Type.Hash(Type.String, Type.String),
     ['files.exclude']                       = Type.Hash(Type.String, Type.Boolean),
     ['editor.semanticHighlighting.enabled'] = Type.Or(Type.Boolean, Type.String),
     ['editor.acceptSuggestionOnEnter']      = Type.String  >> 'on',
 }
 
-local config    = {}
-local rawConfig = {}
-
+---@class config.api
 local m = {}
 m.watchList = {}
 
-local function update(key, value, raw)
-    local oldValue = config[key]
-    config[key]    = value
-    rawConfig[key] = raw
-    m.event(key, value, oldValue)
+m.NULL = {}
+
+m.nullSymbols = {
+    [m.NULL] = true,
+}
+
+---@param scp      scope
+---@param key      string
+---@param nowValue any
+---@param rawValue any
+local function update(scp, key, nowValue, rawValue)
+    local now = m.getNowTable(scp)
+    local raw = m.getRawTable(scp)
+
+    now[key] = nowValue
+    raw[key] = rawValue
 end
 
-function m.set(key, value)
+---@param uri uri
+---@param key? string
+---@return scope
+local function getScope(uri, key)
+    local raw = m.getRawTable(scope.override)
+    if raw then
+        if not key or raw[key] ~= nil then
+            return scope.override
+        end
+    end
+    if uri then
+        ---@type scope
+        local scp = scope.getFolder(uri) or scope.getLinkedScope(uri)
+        if scp then
+            if not key
+            or m.getRawTable(scp)[key] ~= nil then
+                return scp
+            end
+        end
+    end
+    return scope.fallback
+end
+
+---@param scp   scope
+---@param key   string
+---@param value any
+function m.setByScope(scp, key, value)
     local unit = Template[key]
     if not unit then
         return false
     end
-    if util.equal(rawConfig[key], value) then
+    local raw = m.getRawTable(scp)
+    if util.equal(raw[key], value) then
         return false
     end
     if unit:checker(value) then
-        update(key, unit:loader(value), value)
+        update(scp, key, unit:loader(value), value)
     else
-        update(key, unit.default, unit.default)
+        update(scp, key, unit.default, unit.default)
     end
     return true
 end
 
-function m.add(key, value)
+---@param uri   uri
+---@param key   string
+---@param value any
+function m.set(uri, key, value)
+    local scp = getScope(uri)
+    local oldValue = m.get(uri, key)
+    m.setByScope(scp, key, value)
+    local newValue = m.get(uri, key)
+    if not util.equal(oldValue, newValue) then
+        m.event(uri, key, newValue, oldValue)
+        return true
+    end
+    return false
+end
+
+function m.add(uri, key, value)
     local unit = Template[key]
     if not unit then
         return false
     end
-    local list = rawConfig[key]
+    local list = m.getRaw(uri, key)
     if type(list) ~= 'table' then
         return false
     end
@@ -253,18 +319,22 @@ function m.add(key, value)
         copyed[i] = v
     end
     copyed[#copyed+1] = value
-    if unit:checker(copyed) then
-        update(key, unit:loader(copyed), copyed)
+    local oldValue = m.get(uri, key)
+    m.set(uri, key, copyed)
+    local newValue = m.get(uri, key)
+    if not util.equal(oldValue, newValue) then
+        m.event(uri, key, newValue, oldValue)
+        return true
     end
-    return true
+    return false
 end
 
-function m.prop(key, prop, value)
+function m.prop(uri, key, prop, value)
     local unit = Template[key]
     if not unit then
         return false
     end
-    local map = rawConfig[key]
+    local map = m.getRaw(uri, key)
     if type(map) ~= 'table' then
         return false
     end
@@ -276,101 +346,133 @@ function m.prop(key, prop, value)
         copyed[k] = v
     end
     copyed[prop] = value
-    if unit:checker(copyed) then
-        update(key, unit:loader(copyed), copyed)
+    local oldValue = m.get(uri, key)
+    m.set(uri, key, copyed)
+    local newValue = m.get(uri, key)
+    if not util.equal(oldValue, newValue) then
+        m.event(uri, key, newValue, oldValue)
+        return true
     end
-    return true
+    return false
 end
 
-function m.get(key)
-    return config[key]
+---@param uri uri
+---@param key string
+---@return any
+function m.get(uri, key)
+    local scp = getScope(uri, key)
+    local value = m.getNowTable(scp)[key]
+    if value == nil then
+        value = Template[key].default
+    end
+    if value == m.NULL then
+        value = nil
+    end
+    return value
 end
 
-function m.dump()
-    local dump = {}
-
-    local function expand(parent, key, value)
-        local left, right = key:match '([^%.]+)%.(.+)'
-        if left then
-            if not parent[left] then
-                parent[left] = {}
-            end
-            expand(parent[left], right, value)
-        else
-            parent[key] = value
-        end
+---@param uri uri
+---@param key string
+---@return any
+function m.getRaw(uri, key)
+    local scp = getScope(uri, key)
+    local value = m.getRawTable(scp)[key]
+    if value == nil then
+        value = Template[key].default
     end
-
-    for key, value in pairs(config) do
-        expand(dump, key, value)
+    if value == m.NULL then
+        value = nil
     end
-
-    return dump
+    return value
 end
 
-function m.update(new)
+---@param scp  scope
+function m.getNowTable(scp)
+    return scp:get 'config.now'
+        or scp:set('config.now', {})
+end
+
+---@param scp  scope
+function m.getRawTable(scp)
+    return scp:get 'config.raw'
+        or scp:set('config.raw', {})
+end
+
+---@param scp  scope
+---@param ...  table
+function m.update(scp, ...)
+    local oldConfig = m.getNowTable(scp)
+    local newConfig = {}
+    scp:set('config.now', newConfig)
+    scp:set('config.raw', {})
+
     local function expand(t, left)
         for key, value in pairs(t) do
             local fullKey = key
             if left then
                 fullKey = left .. '.' .. key
             end
-            if Template[fullKey] then
-                m.set(fullKey, value)
+            if m.nullSymbols[value] then
+                value = m.NULL
+            end
+            if     Template[fullKey] then
+                m.setByScope(scp, fullKey, value)
             elseif Template['Lua.' .. fullKey] then
-                m.set('Lua.' .. fullKey, value)
+                m.setByScope(scp, 'Lua.' .. fullKey, value)
             elseif type(value) == 'table' then
                 expand(value, fullKey)
             end
         end
     end
 
-    expand(new)
+    local news = table.pack(...)
+    for i = 1, news.n do
+        if news[i] then
+            expand(news[i])
+        end
+    end
+
+    -- compare then fire event
+    if oldConfig then
+        for key, oldValue in pairs(oldConfig) do
+            local newValue = newConfig[key]
+            if not util.equal(oldValue, newValue) then
+                m.event(scp.uri, key, newValue, oldValue)
+            end
+        end
+    end
+
+    m.event(scp.uri, '')
 end
 
----@param callback fun(key: string, value: any, oldValue: any)
+---@param callback fun(uri: uri, key: string, value: any, oldValue: any)
 function m.watch(callback)
     m.watchList[#m.watchList+1] = callback
 end
 
-function m.event(key, value, oldValue)
-    if not m.delay then
-        m.delay = {}
+function m.event(uri, key, value, oldValue)
+    if not m.changes then
+        m.changes = {}
         timer.wait(0, function ()
-            local delay = m.delay
-            m.delay = nil
+            local delay = m.changes
+            m.changes = nil
             for _, info in ipairs(delay) do
                 for _, callback in ipairs(m.watchList) do
-                    callback(info.key, info.value, info.oldValue)
+                    callback(info.uri, info.key, info.value, info.oldValue)
                 end
             end
         end)
     end
-    m.delay[#m.delay+1] = {
+    m.changes[#m.changes+1] = {
+        uri      = uri,
         key      = key,
         value    = value,
         oldValue = oldValue,
     }
 end
 
----@param source config.source
-function m.setSource(source)
-    m._source = source
-end
-
----@return config.source
-function m.getSource()
-    return m._source
-end
-
-function m.init()
-    if m.inited then
-        return
-    end
-    m.inited = true
-    for key, unit in pairs(Template) do
-        m.set(key, unit.default)
-    end
+function m.addNullSymbol(null)
+    m.nullSymbols[null] = true
 end
 
 return m
