@@ -4,15 +4,14 @@ local guide      = require 'parser.guide'
 local parser     = require 'parser.newparser'
 
 local TokenTypes, TokenStarts, TokenFinishs, TokenContents, TokenMarks
-local Ci, Offset, pushError, NextComment, Lines
+local Ci, Offset, pushWarning, NextComment, Lines
 local parseType
 local Parser = re.compile([[
 Main                <-  (Token / Sp)*
 Sp                  <-  %s+
 X16                 <-  [a-fA-F0-9]
-Word                <-  [a-zA-Z0-9_]
 Token               <-  Integer / Name / String / Symbol
-Name                <-  ({} {[a-zA-Z_0-9] [a-zA-Z0-9_.*-]*} {})
+Name                <-  ({} {%name} {})
                     ->  Name
 Integer             <-  ({} {[0-9]+} !'.' {})
                     ->  Integer
@@ -45,7 +44,7 @@ EChar               <-  'a' -> ea
                     /   ('z' (%nl / %s)*)     -> ''
                     /   ('x' {X16 X16})       -> Char16
                     /   ([0-9] [0-9]? [0-9]?) -> Char10
-                    /   ('u{' {Word*} '}')    -> CharUtf8
+                    /   ('u{' {X16*} '}')    -> CharUtf8
 Symbol              <-  ({} {
                             [:|,<>()?+#`{}]
                         /   '[]'
@@ -63,6 +62,7 @@ Symbol              <-  ({} {
     er = '\r',
     et = '\t',
     ev = '\v',
+    name = (m.R('az', 'AZ', '09', '\x80\xff') + m.S('_')) * (m.R('az', 'AZ', '__', '09', '\x80\xff') + m.S('_.*-'))^0,
     Char10 = function (char)
         char = tonumber(char)
         if not char or char < 0 or char > 255 then
@@ -204,7 +204,7 @@ local function nextSymbolOrError(symbol)
         nextToken()
         return true
     end
-    pushError {
+    pushWarning {
         type   = 'LUADOC_MISS_SYMBOL',
         start  = getFinish(),
         finish = getFinish(),
@@ -229,7 +229,7 @@ local function parseIndexField(tp, parent)
     local indexTP, index = nextToken()
     if  indexTP ~= 'integer'
     and indexTP ~= 'string' then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_INDEX_MUST_INT',
             start  = getStart(),
             finish = getFinish(),
@@ -249,7 +249,7 @@ local function parseClass(parent)
     }
     result.class = parseName('doc.class.name', result)
     if not result.class then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_MISS_CLASS_NAME',
             start  = getFinish(),
             finish = getFinish(),
@@ -258,15 +258,7 @@ local function parseClass(parent)
     end
     result.start  = getStart()
     result.finish = getFinish()
-    if not peekToken() then
-        return result
-    end
     if not checkToken('symbol', ':', 1) then
-        pushError {
-            type   = 'LUADOC_MISS_EXTENDS_SYMBOL',
-            start  = result.finish + 1,
-            finish = getStart(),
-        }
         return result
     end
     nextToken()
@@ -276,7 +268,7 @@ local function parseClass(parent)
     while true do
         local extend = parseName('doc.extends.name', result)
         if not extend then
-            pushError {
+            pushWarning {
                 type   = 'LUADOC_MISS_CLASS_EXTENDS_NAME',
                 start  = getFinish(),
                 finish = getFinish(),
@@ -342,6 +334,21 @@ local function parseTypeUnitTable(parent, node)
     return result
 end
 
+local function parseDots(tp, parent)
+    if not checkToken('symbol', '...', 1) then
+        return
+    end
+    nextToken()
+    local dots = {
+        type   = tp,
+        start  = getStart(),
+        finish = getFinish(),
+        parent = parent,
+        [1]    = '...',
+    }
+    return dots
+end
+
 local function  parseTypeUnitFunction()
     local typeUnit = {
         type    = 'doc.type.function',
@@ -361,47 +368,29 @@ local function  parseTypeUnitFunction()
             type   = 'doc.type.arg',
             parent = typeUnit,
         }
-        if checkToken('symbol', '...', 1) then
-            nextToken()
-            local vararg = {
-                type   = 'doc.type.name',
-                start  = getStart(),
+        arg.name = parseName('doc.type.arg.name', arg)
+                or parseDots('doc.type.arg.name', arg)
+        if not arg.name then
+            pushWarning {
+                type   = 'LUADOC_MISS_ARG_NAME',
+                start  = getFinish(),
                 finish = getFinish(),
-                parent = arg,
-                [1]    = '...',
             }
-            arg.name   = vararg
-            if not arg.start then
-                arg.start = arg.name.start
-            end
-            arg.finish = getFinish()
-        else
-            arg.name = parseName('doc.type.name', arg)
-            if not arg.name then
-                pushError {
-                    type   = 'LUADOC_MISS_ARG_NAME',
-                    start  = getFinish(),
-                    finish = getFinish(),
-                }
-                break
-            end
-            if not arg.start then
-                arg.start = arg.name.start
-            end
-            if checkToken('symbol', '?', 1) then
-                nextToken()
-                arg.optional = true
-            end
-            arg.finish = getFinish()
-            if not nextSymbolOrError(':') then
-                break
-            end
-            arg.extends = parseType(arg)
-            if not arg.extends then
-                break
-            end
-            arg.finish = getFinish()
+            break
         end
+        if not arg.start then
+            arg.start = arg.name.start
+        end
+        if checkToken('symbol', '?', 1) then
+            nextToken()
+            arg.optional = true
+        end
+        arg.finish = getFinish()
+        if checkToken('symbol', ':', 1) then
+            nextToken()
+            arg.extends = parseType(arg)
+        end
+        arg.finish = getFinish()
         typeUnit.args[#typeUnit.args+1] = arg
         if checkToken('symbol', ',', 1) then
             nextToken()
@@ -454,7 +443,7 @@ local function parseTypeUnitLiteralTable()
             field.name = parseName('doc.field.name', field)
                     or   parseIndexField('doc.field.name', field)
             if not field.name then
-                pushError {
+                pushWarning {
                     type   = 'LUADOC_MISS_FIELD_NAME',
                     start  = getFinish(),
                     finish = getFinish(),
@@ -491,13 +480,35 @@ local function parseTypeUnitLiteralTable()
     return typeUnit
 end
 
-local function parseTypeUnit(parent, content)
-    local result
-    if content == 'fun' then
-        result = parseTypeUnitFunction()
+local parseTypeUnit
+
+local function parseDocFunction(parent, content)
+    if content == 'async' then
+        local pos = getStart()
+        local tp, cont = peekToken()
+        if tp == 'name' then
+            if cont == 'fun' then
+                nextToken()
+                local func = parseTypeUnit(parent, cont)
+                if func then
+                    func.async = true
+                    func.asyncPos = pos
+                    return func
+                end
+            end
+        end
     end
-    if content == '{' then
-        result = parseTypeUnitLiteralTable()
+    if content == 'fun' then
+        return parseTypeUnitFunction()
+    end
+end
+
+function parseTypeUnit(parent, content)
+    local result = parseDocFunction(parent, content)
+    if not result then
+        if content == '{' then
+            result = parseTypeUnitLiteralTable()
+        end
     end
     if not result then
         result = {
@@ -540,7 +551,7 @@ local function parseResume(parent)
 
     local tp = peekToken()
     if tp ~= 'string' then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_MISS_STRING',
             start  = getFinish(),
             finish = getFinish(),
@@ -559,8 +570,6 @@ function parseType(parent)
         type    = 'doc.type',
         parent  = parent,
         types   = {},
-        enums   = {},
-        resumes = {},
     }
     while true do
         local tp, content = peekToken()
@@ -606,7 +615,7 @@ function parseType(parent)
                 parent = result,
                 [1]    = content,
             }
-            result.enums[#result.enums+1] = typeEnum
+            result.types[#result.types+1] = typeEnum
             if not result.start then
                 result.start = typeEnum.start
             end
@@ -674,7 +683,7 @@ function parseType(parent)
                         else
                             resume.comment = nextComm.text:match('#%s*(.+)', 3)
                         end
-                        result.resumes[#result.resumes+1] = resume
+                        result.types[#result.types+1] = resume
                         result.finish = resume.finish
                     end
                     comments = nil
@@ -706,8 +715,8 @@ function parseType(parent)
         while pushResume() do end
     end
 
-    if #result.types == 0 and #result.enums == 0 and #result.resumes == 0 then
-        pushError {
+    if #result.types == 0 then
+        pushWarning {
             type   = 'LUADOC_MISS_TYPE_NAME',
             start  = getFinish(),
             finish = getFinish(),
@@ -723,7 +732,7 @@ local function parseAlias()
     }
     result.alias = parseName('doc.alias.name', result)
     if not result.alias then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_MISS_ALIAS_NAME',
             start  = getFinish(),
             finish = getFinish(),
@@ -733,7 +742,7 @@ local function parseAlias()
     result.start  = getStart()
     result.extends = parseType(result)
     if not result.extends then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_MISS_ALIAS_EXTENDS',
             start  = getFinish(),
             finish = getFinish(),
@@ -749,8 +758,9 @@ local function parseParam()
         type   = 'doc.param',
     }
     result.param = parseName('doc.param.name', result)
+                or parseDots('doc.param.name', result)
     if not result.param then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_MISS_PARAM_NAME',
             start  = getFinish(),
             finish = getFinish(),
@@ -765,7 +775,7 @@ local function parseParam()
     result.finish = getFinish()
     result.extends = parseType(result)
     if not result.extends then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_MISS_PARAM_EXTENDS',
             start  = getFinish(),
             finish = getFinish(),
@@ -828,7 +838,7 @@ local function parseField()
     result.field = parseName('doc.field.name', result)
                 or parseIndexField('doc.field.name', result)
     if not result.field then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_MISS_FIELD_NAME',
             start  = getFinish(),
             finish = getFinish(),
@@ -844,7 +854,7 @@ local function parseField()
     end
     result.extends = parseType(result)
     if not result.extends then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_MISS_FIELD_EXTENDS',
             start  = getFinish(),
             finish = getFinish(),
@@ -867,7 +877,7 @@ local function parseGeneric()
         }
         object.generic = parseName('doc.generic.name', object)
         if not object.generic then
-            pushError {
+            pushWarning {
                 type   = 'LUADOC_MISS_GENERIC_NAME',
                 start  = getFinish(),
                 finish = getFinish(),
@@ -899,7 +909,7 @@ local function parseVararg()
     }
     result.vararg = parseType(result)
     if not result.vararg then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_MISS_VARARG_TYPE',
             start  = getFinish(),
             finish = getFinish(),
@@ -913,8 +923,9 @@ end
 
 local function parseOverload()
     local tp, name = peekToken()
-    if tp ~= 'name' or name ~= 'fun' then
-        pushError {
+    if tp ~= 'name'
+    or (name ~= 'fun' and name ~= 'async') then
+        pushWarning {
             type   = 'LUADOC_MISS_FUN_AFTER_OVERLOAD',
             start  = getFinish(),
             finish = getFinish(),
@@ -925,7 +936,7 @@ local function parseOverload()
     local result = {
         type = 'doc.overload',
     }
-    result.overload = parseTypeUnitFunction()
+    result.overload = parseDocFunction(result, name)
     if not result.overload then
         return nil
     end
@@ -959,7 +970,7 @@ local function parseVersion()
     while true do
         local tp, text = nextToken()
         if not tp then
-            pushError {
+            pushWarning {
                 type  = 'LUADOC_MISS_VERSION',
                 start  = getFinish(),
                 finish = getFinish(),
@@ -982,7 +993,7 @@ local function parseVersion()
             tp, text = nextToken()
         end
         if tp ~= 'name' then
-            pushError {
+            pushWarning {
                 type  = 'LUADOC_MISS_VERSION',
                 start  = getStart(),
                 finish = getFinish(),
@@ -1028,7 +1039,7 @@ local function parseDiagnostic()
     }
     local nextTP, mode = nextToken()
     if nextTP ~= 'name' then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_MISS_DIAG_MODE',
             start  = getFinish(),
             finish = getFinish(),
@@ -1042,7 +1053,7 @@ local function parseDiagnostic()
     and mode ~= 'disable-line'
     and mode ~= 'disable'
     and mode ~= 'enable' then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_ERROR_DIAG_MODE',
             start  = result.start,
             finish = result.finish,
@@ -1055,7 +1066,7 @@ local function parseDiagnostic()
         while true do
             local name = parseName('doc.diagnostic.name', result)
             if not name then
-                pushError {
+                pushWarning {
                     type   = 'LUADOC_MISS_DIAG_NAME',
                     start  = getFinish(),
                     finish = getFinish(),
@@ -1089,7 +1100,7 @@ local function parseModule()
         result.finish = getFinish()
         result.smark  = getMark()
     else
-        pushError {
+        pushWarning {
             type   = 'LUADOC_MISS_MODULE_NAME',
             start  = getFinish(),
             finish = getFinish(),
@@ -1098,13 +1109,29 @@ local function parseModule()
     return result
 end
 
+local function parseAsync()
+    return {
+        type   = 'doc.async',
+        start  = getFinish(),
+        finish = getFinish(),
+    }
+end
+
+local function parseNoDiscard()
+    return {
+        type   = 'doc.nodiscard',
+        start  = getFinish(),
+        finish = getFinish(),
+    }
+end
+
 local function convertTokens()
     local tp, text = nextToken()
     if not tp then
         return
     end
     if tp ~= 'name' then
-        pushError {
+        pushWarning {
             type   = 'LUADOC_MISS_CATE_NAME',
             start  = getStart(),
             finish = getFinish(),
@@ -1141,6 +1168,10 @@ local function convertTokens()
         return parseDiagnostic()
     elseif text == 'module' then
         return parseModule()
+    elseif text == 'async' then
+        return parseAsync()
+    elseif text == 'nodiscard' then
+        return parseNoDiscard()
     end
 end
 
@@ -1397,8 +1428,11 @@ return function (state)
         groups = {},
     }
 
-    pushError = state.pushError
-    Lines     = state.lines
+    pushWarning = function (err)
+        err.level = err.level or 'Warning'
+        state.pushError(err)
+    end
+    Lines       = state.lines
 
     local ci = 1
     NextComment = function (offset, peek)
